@@ -2,6 +2,7 @@
  * baratinoo.c - Speech Dispatcher backend for Baratinoo (VoxyGen)
  *
  * Copyright (C) 2016 Brailcom, o.p.s.
+ * Copyright (C) 2019-2020 Samuel Thibault <samuel.thibault@ens-lyon.org>
  *
  * This is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by
@@ -52,6 +53,8 @@
 
 #include <semaphore.h>
 
+#include "module_utils_speak_queue.h"
+
 #ifdef BARATINOO_ABI_IS_STABLE_ENOUGH_FOR_ME
 /* See below why this is problematic.  It can however be useful to get the
  * compiler help to check compatibility */
@@ -116,6 +119,7 @@ typedef enum {
 	BV_UNSUPPORTED = -1,
 	BV_8_1,
 	BV_8_4,
+	BV_8_6,
 	N_SUPPORTED_BARATINOO_VERSIONS
 } SupportedBaratinooVersion;
 
@@ -123,6 +127,7 @@ typedef enum {
 static const BARATINOO_TEXT_ENCODING bv_BARATINOO_UTF8[N_SUPPORTED_BARATINOO_VERSIONS] = {
 	[BV_8_1] = BARATINOO_UTF8__V8_1,
 	[BV_8_4] = BARATINOO_UTF8__V8_4,
+	[BV_8_6] = BARATINOO_UTF8__V8_4,
 };
 #define BARATINOO_UTF8 (bv_BARATINOO_UTF8[baratinoo_engine.supported_version])
 
@@ -138,27 +143,37 @@ enum {
 };
 static const size_t bv_VoiceInfo_offsets[N_SUPPORTED_BARATINOO_VERSIONS][N_VI_MEMBERS] = {
 #define BVIF_MEMBER_DECL(struct, member) [VI_##member] = G_STRUCT_OFFSET(struct, member)
-	[BV_8_1] = {
-		BVIF_MEMBER_DECL(BaratinooVoiceInfo__V8_1, name),
-		BVIF_MEMBER_DECL(BaratinooVoiceInfo__V8_1, language),
-		BVIF_MEMBER_DECL(BaratinooVoiceInfo__V8_1, iso639),
-		BVIF_MEMBER_DECL(BaratinooVoiceInfo__V8_1, iso3166),
-		BVIF_MEMBER_DECL(BaratinooVoiceInfo__V8_1, gender),
-		BVIF_MEMBER_DECL(BaratinooVoiceInfo__V8_1, age),
-	},
-	[BV_8_4] = {
-		BVIF_MEMBER_DECL(BaratinooVoiceInfo__V8_4, name),
-		BVIF_MEMBER_DECL(BaratinooVoiceInfo__V8_4, language),
-		BVIF_MEMBER_DECL(BaratinooVoiceInfo__V8_4, iso639),
-		BVIF_MEMBER_DECL(BaratinooVoiceInfo__V8_4, iso3166),
-		BVIF_MEMBER_DECL(BaratinooVoiceInfo__V8_4, gender),
-		BVIF_MEMBER_DECL(BaratinooVoiceInfo__V8_4, age),
-	},
+#define BVIF_ENTRY(struct) {	BVIF_MEMBER_DECL(struct, name),		\
+				BVIF_MEMBER_DECL(struct, language),	\
+				BVIF_MEMBER_DECL(struct, iso639),	\
+				BVIF_MEMBER_DECL(struct, iso3166),	\
+				BVIF_MEMBER_DECL(struct, gender),	\
+				BVIF_MEMBER_DECL(struct, age),		}
+	[BV_8_1] = BVIF_ENTRY(BaratinooVoiceInfo__V8_1),
+	[BV_8_4] = BVIF_ENTRY(BaratinooVoiceInfo__V8_4),
+	[BV_8_6] = BVIF_ENTRY(BaratinooVoiceInfo__V8_4),
+#undef BVIF_ENTRY
 #undef BVIF_MEMBER_DECL
 };
 #define VOICE_INFO_MEMBER(member_type, struct_p, member) \
 	G_STRUCT_MEMBER(member_type, struct_p, bv_VoiceInfo_offsets[baratinoo_engine.supported_version][VI_##member])
 
+/* BCinputTextBufferNew() gained an additional "uri" parameter in 8.6 */
+typedef BCinputTextBuffer (*bv_BCinputTextBufferNew_t)(BARATINOO_PARSING parsing, BARATINOO_TEXT_ENCODING encoding, int voiceIndex, char *voiceModules);
+typedef BCinputTextBuffer (*bv_BCinputTextBufferNew__V8_6_t)(BARATINOO_PARSING parsing, BARATINOO_TEXT_ENCODING encoding, int voiceIndex, char *voiceModules, const char *uri);
+#define BCinputTextBufferNew__V8_1 BCinputTextBufferNew
+static BCinputTextBuffer BCinputTextBufferNew__V8_6(BARATINOO_PARSING parsing, BARATINOO_TEXT_ENCODING encoding, int voiceIndex, char *voiceModules)
+{
+	bv_BCinputTextBufferNew__V8_6_t func = (bv_BCinputTextBufferNew__V8_6_t) BCinputTextBufferNew;
+	return func(parsing, encoding, voiceIndex, voiceModules, NULL);
+}
+static const bv_BCinputTextBufferNew_t bv_BCinputTextBufferNew[] = {
+	[BV_8_1] = BCinputTextBufferNew__V8_1,
+	[BV_8_4] = BCinputTextBufferNew__V8_1,
+	[BV_8_6] = BCinputTextBufferNew__V8_6,
+};
+#define BCinputTextBufferNew (bv_BCinputTextBufferNew[baratinoo_engine.supported_version])
+
 #endif /* ! BARATINOO_ABI_IS_STABLE_ENOUGH_FOR_ME */
 
 
@@ -191,8 +206,6 @@ typedef struct {
 	 * thread is ready to accept new input.  Otherwise, the thread is in
 	 * the process of synthesizing speech. */
 	BCinputTextBuffer buffer;
-        /* The output signal */
-        BCoutputSignalBuffer output_signal;
 
 	SPDVoice **voice_list;
 
@@ -200,22 +213,15 @@ typedef struct {
 	int voice;
 
 	/* request flags */
-	gboolean pause_requested;
-	gboolean stop_requested;
 	gboolean close_requested;
-
-	SPDMarks marks;
 } Engine;
 
 /* engine and state */
 static Engine baratinoo_engine = {
 	.engine = NULL,
 	.buffer = NULL,
-	.output_signal = NULL,
 	.voice_list = NULL,
 	.voice = 0,
-	.pause_requested = FALSE,
-	.stop_requested = FALSE,
 	.close_requested = FALSE
 };
 
@@ -235,6 +241,8 @@ static void append_ssml_as_proprietary(const Engine *engine, GString *buf, const
 /* Module configuration options */
 MOD_OPTION_1_STR(BaratinooConfigPath);
 MOD_OPTION_1_INT(BaratinooSampleRate);
+MOD_OPTION_1_INT(BaratinooResponsiveness);
+MOD_OPTION_1_INT(BaratinooQueueSize);
 MOD_OPTION_1_INT(BaratinooMinRate);
 MOD_OPTION_1_INT(BaratinooNormalRate);
 MOD_OPTION_1_INT(BaratinooMaxRate);
@@ -268,6 +276,12 @@ int module_load(void)
 	/* Sample rate. 16000Hz is the voices default, not requiring resampling */
 	MOD_OPTION_1_INT_REG(BaratinooSampleRate, 16000);
 
+	/* Let Baratinoo handle by default */
+	MOD_OPTION_1_INT_REG(BaratinooResponsiveness, -1);
+
+	/* Default to 20s queuing */
+	MOD_OPTION_1_INT_REG(BaratinooQueueSize, 20*BaratinooSampleRate);
+
 	/* Speech rate */
 	MOD_OPTION_1_INT_REG(BaratinooMinRate, -100);
 	MOD_OPTION_1_INT_REG(BaratinooNormalRate, 0);
@@ -290,6 +304,7 @@ static SupportedBaratinooVersion get_baratinoo_supported_version(void)
 		case 8: switch (version->minor) {
 			case 1: return BV_8_1;
 			case 4: return BV_8_4;
+			case 6: return BV_8_6;
 		} break;
 	}
 
@@ -312,8 +327,6 @@ int module_init(char **status_info)
 
 	*status_info = NULL;
 
-	engine->pause_requested = FALSE;
-	engine->stop_requested = FALSE;
 	engine->close_requested = FALSE;
 
 	/* Init Baratinoo */
@@ -363,11 +376,7 @@ int module_init(char **status_info)
 
 	/* Setup output (audio) signal handling */
 	DBG(DBG_MODNAME "Using PCM output at %dHz", BaratinooSampleRate);
-	engine->output_signal = BCoutputSignalBufferNew(BARATINOO_PCM, BaratinooSampleRate);
-	if (!engine->output_signal) {
-		  DBG(DBG_MODNAME "Cannot allocate BCoutputSignalBufferNew");
-		  return -1;
-	}
+	BCsetOutputSignal(engine->engine, baratinoo_output_signal, engine, BARATINOO_PCM, BaratinooSampleRate);
 	if (BCgetState(engine->engine) != BARATINOO_INITIALIZED) {
 		DBG(DBG_MODNAME "Failed to initialize output signal handler");
 		*status_info = g_strdup("Failed to initialize Baratinoo output "
@@ -375,7 +384,6 @@ int module_init(char **status_info)
 					"sample rate correct?");
 		return -1;
 	}
-	BCoutputTextBufferSetInEngine(engine->output_signal, engine->engine);
 
 	BCsetWantedEvent(engine->engine, BARATINOO_MARKER_EVENT);
 
@@ -394,7 +402,10 @@ int module_init(char **status_info)
 		return -1;
 	}
 
-	module_marks_init(&engine->marks);
+	if (module_speak_queue_init(BaratinooQueueSize, status_info)) {
+		DBG(DBG_MODNAME "queue creation failed");
+		return -1;
+	}
 
 	DBG(DBG_MODNAME "Initialization successfully.");
 	*status_info = g_strdup("Baratinoo initialized successfully.");
@@ -427,9 +438,6 @@ int module_speak(gchar *data, size_t bytes, SPDMessageType msgtype)
 		return 0;
 	}
 
-	engine->pause_requested = FALSE;
-	engine->stop_requested = FALSE;
-
 	/* select voice following parameters.  we don't use tags for this as
 	 * we need to do some computation on our end anyway and need pass an
 	 * ID when creating the buffer too */
@@ -469,12 +477,26 @@ int module_speak(gchar *data, size_t bytes, SPDMessageType msgtype)
 	}
 
 	switch (msgtype) {
-	case SPD_MSGTYPE_SPELL: /* FIXME: use \spell one day? */
+	case SPD_MSGTYPE_SPELL:	/* FIXME: use \spell when Voxygen actuall implements it */
+				/* TODO: in the meanwhile use a generic engine */
 	case SPD_MSGTYPE_CHAR:
 		g_string_append(buffer, "\\sayas<{characters}");
 		g_string_append_len(buffer, data, bytes);
 		g_string_append(buffer, "\\sayas>{}");
 		break;
+	case SPD_MSGTYPE_KEY:	/* TODO: use a generic engine */
+		if (g_utf8_strlen(data, bytes) == 1) {
+			g_string_append(buffer, "\\sayas<{characters}");
+			g_string_append_len(buffer, data, bytes);
+			g_string_append(buffer, "\\sayas>{}");
+		} else {
+			gchar *c;
+			g_string_append_len(buffer, data, bytes);
+			for (c = buffer->str; *c; c++)
+				if (*c == '_')
+					*c = ' ';
+		}
+		break;
 	default: /* FIXME: */
 	case SPD_MSGTYPE_TEXT:
 		append_ssml_as_proprietary(engine, buffer, data, bytes);
@@ -508,27 +530,16 @@ err:
 
 int module_stop(void)
 {
-	Engine *engine = &baratinoo_engine;
-
 	DBG(DBG_MODNAME "Stop requested");
-	engine->stop_requested = TRUE;
-	module_marks_stop(&engine->marks);
-	if (module_audio_id) {
-		DBG(DBG_MODNAME "Stopping audio currently playing.");
-		if (spd_audio_stop(module_audio_id) != 0)
-			DBG(DBG_MODNAME "spd_audio_stop() returned non-zero value.");
-	}
+	module_speak_queue_stop();
 
 	return 0;
 }
 
 size_t module_pause(void)
 {
-	Engine *engine = &baratinoo_engine;
-
 	DBG(DBG_MODNAME "Pause requested");
-	engine->pause_requested = TRUE;
-	module_marks_stop(&engine->marks);
+	module_speak_queue_pause();
 
 	return 0;
 }
@@ -541,10 +552,10 @@ int module_close(void)
 
 	DBG(DBG_MODNAME "Terminating threads");
 
+	module_speak_queue_terminate();
+
 	/* Politely ask the thread to terminate */
-	engine->stop_requested = TRUE;
 	engine->close_requested = TRUE;
-	module_marks_stop(&engine->marks);
 	sem_post(&engine->semaphore);
 	/* ...and give it a chance to actually quit. */
 	g_usleep(25000);
@@ -570,10 +581,6 @@ int module_close(void)
 		engine->voice_list = NULL;
 	}
 
-	/* destroy output signal */
-	BCoutputSignalBufferDelete(engine->output_signal);
-	engine->output_signal = NULL;
-
 	/* destroy engine */
 	if (engine->engine) {
 	    BCdelete(engine->engine);
@@ -583,7 +590,7 @@ int module_close(void)
 	/* uninitialize */
 	BCterminatelib();
 
-	module_marks_clear(&engine->marks);
+	module_speak_queue_free();
 
 	DBG(DBG_MODNAME "Module closed.");
 
@@ -612,7 +619,6 @@ static SPDVoice **baratinoo_list_voices(BCengine *engine)
     for (i = 0; i < n_voices; i++) {
 	SPDVoice *voice;
 	const char *language;
-	const char *dash;
 	BaratinooVoiceInfo voice_info_DO_NO_ACCESS_DIRECTLY = BCgetVoiceInfo(engine, i);
 	void *voice_info = &voice_info_DO_NO_ACCESS_DIRECTLY;
 
@@ -625,13 +631,7 @@ static SPDVoice **baratinoo_list_voices(BCengine *engine)
 	voice->name = g_strdup(VOICE_INFO_MEMBER(char *, voice_info, name));
 
 	language = VOICE_INFO_MEMBER(char *, voice_info, language);
-	dash = strchr(language, '-');
-	if (dash) {
-	    voice->language = g_strndup(language, dash - language);
-	    voice->variant = g_ascii_strdown(dash + 1, -1);
-	} else {
-	    voice->language = g_strdup(language);
-	}
+	voice->language = g_strdup(language);
 
 	voices[i] = voice;
     }
@@ -640,6 +640,11 @@ static SPDVoice **baratinoo_list_voices(BCengine *engine)
     return voices;
 }
 
+void module_speak_queue_cancel(void)
+{
+	/* We will stop the synth from _baratinoo_speak */
+}
+
 /**
  * @brief Internal TTS thread.
  * @param data An Engine structure.
@@ -648,8 +653,6 @@ static SPDVoice **baratinoo_list_voices(BCengine *engine)
  * The TTS thread.  It waits on @c Engine::semaphore to consume input data
  * from @c Engine::buffer.
  *
- * @see Engine::pause_requested
- * @see Engine::stop_requested
  * @see Engine::close_requested
  */
 static void *_baratinoo_speak(void *data)
@@ -662,7 +665,6 @@ static void *_baratinoo_speak(void *data)
 	while (!engine->close_requested) {
 		sem_wait(&engine->semaphore);
 		DBG(DBG_MODNAME "Semaphore on");
-		engine->stop_requested = FALSE;
 
 		if (!engine->buffer)
 			continue;
@@ -670,67 +672,45 @@ static void *_baratinoo_speak(void *data)
 		state = BCinputTextBufferSetInEngine(engine->buffer, engine->engine);
 		if (state != BARATINOO_READY) {
 			DBG(DBG_MODNAME "Failed to set input buffer");
-			continue;
+			goto cont;
 		}
 
-		module_report_event_begin();
-		while (1) {
-			if (engine->stop_requested || engine->close_requested) {
-				DBG(DBG_MODNAME "Stop in child, terminating");
-				BCinputTextBufferDelete(engine->buffer);
-				engine->buffer = NULL;
-				module_report_event_stop();
-				break;
-			}
+		if (!module_speak_queue_before_synth())
+			goto cont;
 
-			do {
-				state = BCprocessLoop(engine->engine, -1);
-				if (state == BARATINOO_EVENT) {
-					BaratinooEvent event = BCgetEvent(engine->engine);
-					if (event.type == BARATINOO_MARKER_EVENT) {
-						DBG(DBG_MODNAME "Reached mark '%s' at sample %lu", event.data.marker.name, event.sampleStamp);
-						/* TODO: re-enable marks once audio glitches of module_tts_output are fixed */
-						/* module_marks_add(&engine->marks, event.sampleStamp, event.data.marker.name); */
-						/* if reached a spd mark and pausing requested, stop */
-						if (engine->pause_requested &&
-						    g_str_has_prefix(event.data.marker.name, INDEX_MARK_BODY)) {
-							DBG(DBG_MODNAME "Pausing in thread");
-							state = BCpurge(engine->engine);
-							engine->pause_requested = FALSE;
-							module_report_event_pause();
-						}
-					}
-				} else if (state == BARATINOO_INPUT_ERROR ||
-					   state == BARATINOO_ENGINE_ERROR) {
-					/* CANCEL would be better I guess, but
-					 * that's good enough */
-					module_report_event_stop();
-				}
-			} while (state == BARATINOO_RUNNING || state == BARATINOO_EVENT);
-
-			BCinputTextBufferDelete(engine->buffer);
-			engine->buffer = NULL;
-
-			DBG(DBG_MODNAME "Trying to synthesize text");
-			if (BCoutputSignalBufferIsError(engine->output_signal) || engine->close_requested) {
-				DBG(DBG_MODNAME "Error with the output signal");
-				BCoutputSignalBufferResetSignal(engine->output_signal);
-				module_report_event_stop();
-			} else {
-				baratinoo_output_signal(engine, BCoutputSignalBufferGetSignalBuffer(engine->output_signal), BCoutputSignalBufferGetSignalLength(engine->output_signal));
-				BCoutputSignalBufferResetSignal(engine->output_signal);
-				if (engine->stop_requested || engine->close_requested) {
-					DBG(DBG_MODNAME "Stop in child, terminating");
-					module_report_event_stop();
-				} else {
-					module_report_event_end();
-				}
-			}
-			break;
+		if (module_speak_queue_stop_requested() || engine->close_requested) {
+			DBG(DBG_MODNAME "Stop in child, terminating");
+			goto cont;
 		}
-		engine->stop_requested = FALSE;
+
+		module_speak_queue_before_play();
+		do {
+			state = BCprocessLoop(engine->engine, BaratinooResponsiveness);
+			if (state == BARATINOO_EVENT) {
+				BaratinooEvent event = BCgetEvent(engine->engine);
+				if (event.type == BARATINOO_MARKER_EVENT) {
+					DBG(DBG_MODNAME "Reached mark '%s' at sample %lu", event.data.marker.name, event.sampleStamp);
+					module_speak_queue_add_mark(event.data.marker.name);
+				}
+			} else if (state == BARATINOO_INPUT_ERROR ||
+				   state == BARATINOO_ENGINE_ERROR) {
+				module_speak_queue_stop();
+			}
+		} while (state == BARATINOO_RUNNING || state == BARATINOO_EVENT);
+
+		if (module_speak_queue_stop_requested() || engine->close_requested) {
+			DBG(DBG_MODNAME "Stop in child, terminating");
+		} else {
+			DBG(DBG_MODNAME "Finished synthesizing");
+			module_speak_queue_add_end();
+		}
+
+cont:
+		BCinputTextBufferDelete(engine->buffer);
+		engine->buffer = NULL;
 	}
 
+
 	DBG(DBG_MODNAME "leaving thread with state=%d", state);
 
 	pthread_exit(NULL);
@@ -1025,7 +1005,15 @@ static void baratinoo_trace_cb(BaratinooTraceLevel level, int engine_num, const
  */
 static int baratinoo_output_signal(void *private_data, const void *address, int length)
 {
-	Engine *engine = private_data;
+	/* If stop is requested during synthesis, abort here to stop speech as
+	 * early as possible, even if the engine didn't finish its cycle yet. */
+	if (module_speak_queue_stop_requested())
+	{
+		DBG(DBG_MODNAME "Not playing message because it got stopped");
+		return 1;
+	}
+
+	/* Engine *engine = private_data; */
 	AudioTrack track;
 #if defined(BYTE_ORDER) && (BYTE_ORDER == BIG_ENDIAN)
 	AudioFormat format = SPD_AUDIO_BE;
@@ -1033,13 +1021,6 @@ static int baratinoo_output_signal(void *private_data, const void *address, int
 	AudioFormat format = SPD_AUDIO_LE;
 #endif
 
-	/* If stop is requested during synthesis, abort here to stop speech as
-	 * early as possible, even if the engine didn't finish its cycle yet. */
-	if (engine->stop_requested) {
-		DBG(DBG_MODNAME "Not playing message because it got stopped");
-		return engine->stop_requested;
-	}
-
 	/* We receive 16 bits PCM data */
 	track.num_samples = length / 2; /* 16 bits per sample = 2 bytes */
 	track.num_channels = 1;
@@ -1047,13 +1028,10 @@ static int baratinoo_output_signal(void *private_data, const void *address, int
 	track.bits = 16;
 	track.samples = (short *) address;
 
-	DBG(DBG_MODNAME "Playing part of the message");
-	if (module_tts_output_marks(track, format, &engine->marks) < 0)
-		DBG(DBG_MODNAME "ERROR: failed to play the track");
+	DBG(DBG_MODNAME "Queueing %d samples", length / 2);
+	module_speak_queue_add_audio(&track, format);
 
-	module_marks_clear(&engine->marks);
-
-	return engine->stop_requested;
+	return module_speak_queue_stop_requested();
 }
 
 /* SSML conversion functions */
@@ -1199,7 +1177,7 @@ static void ssml2baratinoo_end_element(GMarkupParseContext *ctx,
  * they affect speech as means of intonation and pauses.
  *
  * The approach here is that for every punctuation character included in the
- * selected mode (none/some/all), we wrap it in "\sayas<{characters}" markup
+ * selected mode (none/some/most/all), we wrap it in "\sayas<{characters}" markup
  * so that it is spoken by the engine.  But in order to keep the punctuation
  * meaning of the character, in case it has some, we duplicate it outside the
  * markup with a heuristic on whether it will or not affect speech intonation
@@ -1257,7 +1235,8 @@ static void ssml2baratinoo_text(GMarkupParseContext *ctx,
 
 			/* if punctuation mode is not NONE and the character
 			 * should be spoken, manually wrap it with \sayas */
-			say_as_char = ((msg_settings.punctuation_mode == SPD_PUNCT_SOME &&
+			say_as_char = (((msg_settings.punctuation_mode == SPD_PUNCT_SOME ||
+					 msg_settings.punctuation_mode == SPD_PUNCT_MOST) &&
 					g_utf8_strchr(BaratinooPunctuationList, -1, ch)) ||
 				       (msg_settings.punctuation_mode == SPD_PUNCT_ALL &&
 					g_unichar_ispunct(ch)));
