@@ -43,7 +43,7 @@ static pthread_t generic_speak_thread;
 static pid_t generic_pid;
 static sem_t generic_semaphore;
 
-static char **generic_message;
+static char *generic_message;
 static SPDMessageType generic_message_type;
 
 static int generic_position = 0;
@@ -71,11 +71,13 @@ void generic_set_punct(SPDPunctuation punct);
 
 MOD_OPTION_1_STR(GenericExecuteSynth)
     MOD_OPTION_1_STR(GenericCmdDependency)
+    MOD_OPTION_1_STR(GenericSoundIconFolder)
 
     MOD_OPTION_1_INT(GenericMaxChunkLength)
     MOD_OPTION_1_STR(GenericDelimiters)
     MOD_OPTION_1_STR(GenericPunctNone)
     MOD_OPTION_1_STR(GenericPunctSome)
+    MOD_OPTION_1_STR(GenericPunctMost)
     MOD_OPTION_1_STR(GenericPunctAll)
     MOD_OPTION_1_STR(GenericStripPunctChars)
     MOD_OPTION_1_STR(GenericRecodeFallback)
@@ -110,6 +112,7 @@ int module_load(void)
 
 	MOD_OPTION_1_STR_REG(GenericExecuteSynth, "");
 	MOD_OPTION_1_STR_REG(GenericCmdDependency, "");
+	MOD_OPTION_1_STR_REG(GenericSoundIconFolder, "/usr/share/sounds/sound-icons/");
 
 	REGISTER_DEBUG();
 
@@ -138,6 +141,7 @@ int module_load(void)
 
 	MOD_OPTION_1_STR_REG(GenericPunctNone, "");
 	MOD_OPTION_1_STR_REG(GenericPunctSome, "");
+	MOD_OPTION_1_STR_REG(GenericPunctMost, "");
 	MOD_OPTION_1_STR_REG(GenericPunctAll, "");
 
 	module_register_available_voices();
@@ -159,11 +163,11 @@ int module_init(char **status_info)
 
 	generic_msg_language =
 	    (TGenericLanguage *) g_malloc(sizeof(TGenericLanguage));
-	generic_msg_language->code = g_strdup("en");
+	generic_msg_language->code = g_strdup("en-US");
 	generic_msg_language->charset = g_strdup("iso-8859-1");
 	generic_msg_language->name = g_strdup("english");
 
-	generic_message = g_malloc(sizeof(char *));
+	generic_message = NULL;
 
 	sem_init(&generic_semaphore, 0, 0);
 
@@ -230,17 +234,18 @@ int module_speak(gchar * data, size_t bytes, SPDMessageType msgtype)
 	if (tmp == NULL)
 		return -1;
 
+	/* TODO: use a generic engine for SPELL, CHAR, KEY */
 	if (msgtype == SPD_MSGTYPE_TEXT)
-		*generic_message = module_strip_ssml(tmp);
+		generic_message = module_strip_ssml(tmp);
 	else
-		*generic_message = g_strdup(tmp);
+		generic_message = g_strdup(tmp);
 	g_free(tmp);
 
-	module_strip_punctuation_some(*generic_message, GenericStripPunctChars);
+	module_strip_punctuation_some(generic_message, GenericStripPunctChars);
 
-	generic_message_type = SPD_MSGTYPE_TEXT;
+	generic_message_type = msgtype;
 
-	DBG("Requested data: |%s|\n", data);
+	DBG("Requested data (%d): |%s|\n", msgtype, data);
 
 	/* Send semaphore signal to the speaking thread */
 	generic_speaking = 1;
@@ -354,6 +359,28 @@ void *_generic_speak(void *nothing)
 		sem_wait(&generic_semaphore);
 		DBG("Semaphore on\n");
 
+		const char *play_command = NULL;
+		play_command = spd_audio_get_playcmd(module_audio_id);
+
+		if (play_command == NULL) {
+			DBG("This audio backend has no default play command; using \"play\"\n");
+			play_command = "play";
+		}
+
+		if (generic_message_type == SPD_MSGTYPE_SOUND_ICON) {
+			if (strchr(generic_message, '\\') ||
+			    strchr(generic_message, '\'') ||
+			    strchr(generic_message, '/')) {
+				DBG("Warning: bad icon name %s\n", generic_message);
+			}
+			char *cmd = g_strdup_printf("%s '%s/%s'", play_command, GenericSoundIconFolder, generic_message);
+			system(cmd);
+			DBG("icon command = |%s|\n", cmd);
+			free(cmd);
+			generic_speaking = 0;
+			continue;
+		}
+
 		ret = pipe(module_pipe.pc);
 		if (ret != 0) {
 			DBG("Can't create pipe pc\n");
@@ -390,7 +417,6 @@ void *_generic_speak(void *nothing)
 				char *p;
 				char *tmpdir, *homedir;
 				const char *helper;
-				const char *play_command = NULL;
 
 				helper = getenv("TMPDIR");
 				if (helper)
@@ -405,13 +431,6 @@ void *_generic_speak(void *nothing)
 					homedir =
 					    g_strdup("UNKNOWN_HOME_DIRECTORY");
 
-				play_command =
-				    spd_audio_get_playcmd(module_audio_id);
-				if (play_command == NULL) {
-					DBG("This audio backend has no default play command; using \"play\"\n");
-					play_command = "play";
-				}
-
 				/* Set this process as a process group leader (so that SIGKILL
 				   is also delivered to the child processes created by system()) */
 				if (setpgid(0, 0) == -1)
@@ -478,7 +497,7 @@ void *_generic_speak(void *nothing)
 			/* This is the parent. Send data to the child. */
 
 			generic_position =
-			    module_parent_wfork(module_pipe, *generic_message,
+			    module_parent_wfork(module_pipe, generic_message,
 						generic_message_type,
 						GenericMaxChunkLength,
 						GenericDelimiters,
@@ -641,9 +660,17 @@ void generic_set_volume(int volume)
 
 void generic_set_language(char *lang)
 {
-
+	DBG("Setting language %s", lang);
+	char *dash = strchr(lang, '-');
 	generic_msg_language =
 	    (TGenericLanguage *) get_ht_option(GenericLanguage, lang);
+	if (generic_msg_language == NULL && dash) {
+		char *lang_only = g_strdup(lang);
+		lang_only[dash-lang] = 0;
+		generic_msg_language =
+		    (TGenericLanguage *) get_ht_option(GenericLanguage, lang_only);
+		g_free(lang_only);
+	}
 	if (generic_msg_language == NULL) {
 		DBG("Language %s not found in the configuration file, using defaults.", lang);
 		generic_msg_language =
@@ -658,7 +685,7 @@ void generic_set_language(char *lang)
 		    lang);
 		generic_msg_language =
 		    (TGenericLanguage *) g_malloc(sizeof(TGenericLanguage));
-		generic_msg_language->code = g_strdup("en");
+		generic_msg_language->code = g_strdup("en-US");
 		generic_msg_language->charset = g_strdup("iso-8859-1");
 		generic_msg_language->name = g_strdup("english");
 	}
@@ -668,8 +695,6 @@ void generic_set_language(char *lang)
 
 void generic_set_voice(SPDVoiceType voice)
 {
-	if (generic_msg_voice_str != NULL)
-		return;
 	assert(generic_msg_language);
 	generic_msg_voice_str =
 	    module_getvoice(generic_msg_language->code, voice);
@@ -693,6 +718,13 @@ void generic_set_punct(SPDPunctuation punct)
 	} else if (punct == SPD_PUNCT_SOME) {
 		generic_msg_punct_str = g_strdup((char *)GenericPunctSome);
 		return;
+	} else if (punct == SPD_PUNCT_MOST) {
+		if (GenericPunctMost[0] == '\0' && GenericPunctSome[0] != '\0')
+			/* Compatibility with old configuration files */
+			generic_msg_punct_str = g_strdup((char *)GenericPunctSome);
+		else
+			generic_msg_punct_str = g_strdup((char *)GenericPunctMost);
+		return;
 	} else if (punct == SPD_PUNCT_ALL) {
 		generic_msg_punct_str = g_strdup((char *)GenericPunctAll);
 		return;
