@@ -14,9 +14,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public License
- * along with this package; see the file COPYING.  If not, write to the Free
- * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  * $Id: module_main.c,v 1.17 2008-10-15 17:05:37 hanke Exp $
  */
@@ -77,8 +75,6 @@ int main(int argc, char *argv[])
 	char *configfilename = NULL;
 	char *status_info = NULL;
 
-	g_thread_init(NULL);
-
 	/* Initialize ltdl's list of preloaded audio backends. */
 	LTDL_SET_PRELOADED_SYMBOLS();
 	module_num_dc_options = 0;
