@@ -14,9 +14,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this package; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
- * Boston, MA 02110-1301, USA.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  *
  * $Id: output.h,v 1.9 2008-06-27 12:28:50 hanke Exp $
  */
@@ -26,7 +24,7 @@
 
 OutputModule *get_output_module(const TSpeechDMessage * message);
 
-int output_speak(TSpeechDMessage * msg);
+int output_speak(TSpeechDMessage * msg, OutputModule *output);
 int output_stop();
 size_t output_pause();
 int output_is_speaking(char **index_mark);
