@@ -112,6 +112,7 @@ queue_message(TSpeechDMessage * new, int fd, int history_flag,
 		COPY_SET_STR(audio_alsa_device);
 		COPY_SET_STR(audio_nas_server);
 		COPY_SET_STR(audio_pulse_server);
+		COPY_SET_STR(audio_pulse_device);
 
 		/* And we set the global id (note that this is really global, not
 		 * depending on the particular client, but unique) */
@@ -258,7 +259,7 @@ int serve(int fd)
 		}
 
 		/* Parse the data and read the reply */
-		MSG2(5, "protocol", "%d:DATA:|%s| (%d)", fd, buf, bytes);
+		MSG2(5, "protocol", "%d:DATA:|%s| (%lu)", fd, buf, (unsigned long) bytes);
 		reply = parse(buf, bytes, fd);
 		g_free(buf);
 	}
