@@ -61,10 +61,43 @@ void destroy_pid_file();
 int server_socket;
 
 GMainLoop *main_loop = NULL;
-gint server_timeout_source = 0;
+gint server_timeout_source = -1;
 
 int client_count = 0;
 
+struct SpeechdOptions SpeechdOptions;
+struct SpeechdStatus SpeechdStatus;
+
+pthread_t speak_thread;
+pthread_mutex_t logging_mutex;
+pthread_mutex_t element_free_mutex;
+pthread_mutex_t output_layer_mutex;
+pthread_mutex_t socket_com_mutex;
+
+GHashTable *fd_settings;
+GHashTable *language_default_modules;
+GHashTable *fd_uid;
+
+TSpeechDQueue *MessageQueue;
+GList *MessagePausedList;
+
+GList *client_specific_settings;
+
+GList *last_p5_block;
+
+TFDSetElement GlobalFDSet;
+
+int speaking_pipe[2];
+
+GHashTable *speechd_sockets_status;
+
+FILE *logfile;
+FILE *custom_logfile;
+char *custom_log_kind;
+FILE *debug_logfile;
+
+TSpeechDMode spd_mode;
+
 static gboolean speechd_client_terminate(gpointer key, gpointer value, gpointer user);
 static gboolean speechd_reload_dead_modules(gpointer user_data);
 static gboolean speechd_load_configuration(gpointer user_data);
@@ -949,13 +982,15 @@ void check_client_count(void)
 	if (client_count <= 0
 	    && SpeechdOptions.server_timeout > 0) {
 		MSG(4, "Currently no clients connected, enabling shutdown timer.");
-		server_timeout_source = 
-		                        g_timeout_add_seconds(
+		server_timeout_source = g_timeout_add_seconds(
 		                        SpeechdOptions.server_timeout,
 		                        speechd_quit, NULL);
 	} else {
-	MSG(4, "Clients connected, disabling shutdown timer.");
-		g_source_remove(server_timeout_source);
+		if (server_timeout_source >= 0) {
+			MSG(4, "Clients connected, disabling shutdown timer.");
+			g_source_remove(server_timeout_source);
+			server_timeout_source = -1;
+		}
 	}
 }
 
